structure RandomAI :> PLAYER = struct

    open AIGame
    type player = Types.move * Time.time -> Types.move

    fun getName() = "RandomAI"

    val date = Date.fromTimeLocal(Time.now())
    val seedset = Random.rand(Date.second(date), Date.minute(date))

    fun getNextMove (boardSize: int ref)
                    (game: game ref)
                    (color: Types.color ref)
                    (mv: Types.move, timeLeft: Time.time): Types.move =
      let
        val center = !boardSize div 2
        val random = Random.randRange(0,!boardSize - 1)
        val (x,y)  = 
          case mv of
             Types.Move(x,y) => (x,y)
           | Types.Start     => (~1, ~1)
           |     _     => raise Fail "incorrect move!"

        fun otherMoves() =
          let
            val {gameBoard, moveList,...} = !game
            val x = random(seedset)
            val y = random(seedset)
          in
            if (validMove(Types.Move(x, y),!game))
            then Types.Move(x,y)
            else otherMoves()
          end

        fun updateGame(c : Types.move) =
          let
            val g = step(!game,c)
          in
            (game := g;
             c)
          end
      in
        (* if last move was Start then put piece in center *)
        (* else random :)*)
        updateGame(if (mv = Types.Start) then Types.Move(center,center) 
                   else (updateGame(mv); otherMoves()))
      end

    fun createPlayer (p:Types.color)
                     (boardSize:int)
                     (timeLeft:Time.time) =
      let
        val game = ref (newGame(boardSize, timeLeft))
        val boardSize = ref boardSize
        val color = ref p
      in
        getNextMove (boardSize) (game) (color)
      end
end
